import yaml
import toga
from toga_layout import Layout
from toga.style.pack import Pack, COLUMN, ROW

import styles
import events


def button_handler(widget):
    print("hello")


def build(app):
    layout = Layout()
    layout.styles = [styles]
    layout.events = [events]
    return layout.load('./example/layout.yaml')

def main():
    return toga.App('Calculator', 'org.beeware.helloworld', startup=build)


if __name__ == '__main__':
    main().main_loop()



import toga
import yaml


class Layout:

    def __init__(self, layout = None):
        self._styles = []
        self._events = []

    def load(self, layout_path):
        layout_file = open(layout_path, 'r')
        with layout_file as f:
            self.layout = yaml.load(f.read())
            self.widgets = self._create_layout(self.layout)
            return self.widgets

    @property
    def styles(self):
        return self._styles

    @styles.setter
    def styles(self, styles):
        self._styles = styles

    @property
    def events(self):
        return self._events

    @events.setter
    def events(self, events):
        self._events = events

    def _create_layout(self, layout, parent=None):
        """ recursive and dinamically creates a toga layout

        params:
        - layout: tree dict
        - parent: parent widget
        """

        # structure checks
        if len(layout.keys()) != 1:
            breakpoint()
            # node of the tree
            raise Exception(f'Bad Format near: {layout}')

        # get id of widget
        name = list(layout.keys())[0]

        # get current yaml node to parse
        current = layout[name]

        # get toga class
        widget_class = getattr(toga, current.pop('type', None))

        # extract special parameters for toga.WIDGET.add() function
        # TODO: improve this
        add_params = []
        if isinstance(parent, toga.OptionContainer):
            add_params.append(current.pop('label', None))

        # get children
        children = current.pop('widgets', None)

        # parse positional parameters
        positional = current.pop('args', None)

        # parse optional parameters
        optional = self._build_optional_parameters(current.items())

        # create toga widget
        # TODO: Improve this if
        if positional:
            widget = widget_class(*positional, id=name, **optional)
        else:
            widget = widget_class(id=name, **optional)

        add_params.append(widget)
    
        # create layout for children
        if children:
            for child in children:
                self._create_layout(child, widget)

        # add widget to parent
        if parent:
            if type(parent) == toga.ScrollContainer:
                #special case of scroll container, only needs one widget as child 
                parent.content = widget
            else:
                parent.add(*add_params)
 
        return widget

    def _build_optional_parameters(self, items):
        
        optional = {key: value for (key, value) in items if key not in ['args', 'type', 'widgets']}
        
        # sanitase optional parameters
        for key, value in optional.items():
            
            # params starting with _on are converted to a function
            if 'on_' in key:
                for event_lib in self._events:
                    if getattr(event_lib, value, None):
                        # found event and exit
                        optional[key] = getattr(event_lib, value)
                        break

            # look for a style in style libs
            if key == 'style':
                for style_lib in self._styles:
                    if getattr(style_lib, value, None):
                        optional['style'] = getattr(style_lib, value)
                        break
        
        return optional


from toga.style.pack import Pack, COLUMN, ROW

full_width = Pack(flex=1)
full_column = Pack(flex=1,direction=COLUMN)
full_row = Pack(flex=1,direction=ROW)
main_box_style = Pack(padding=10, direction=COLUMN, flex=1)
grid_row_btn = Pack(direction=ROW, flex=1)
generic_style = Pack(padding=10, flex=1)
btn_number = Pack(padding=5, flex=1)






from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="toga-layout-builder", 
    version="0.1.0",
    author="Ignacio Cabeza",
    author_email="ignaciocabeza@gmail.com",
    description="Toga Layout",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ignaciocabeza/toga-layout-builder",
    packages=find_packages(exclude=("tests",)),
    install_requires=[
        'toga',
        'pyyaml',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)